@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2010-2015  Tetsuo Handa <penguin-kernel@I-love.SAKURA.ne.jp>
  *
- * Version: 1.0.35   2015/11/11
+ * Version: 1.0.36   2017/02/20
  */
 
 #include "internal.h"
@@ -11,10 +11,24 @@
 
 /* Prototype definition. */
 
+static int __ccs_alloc_task_security(const struct task_struct *task);
+static void __ccs_free_task_security(const struct task_struct *task);
+
 /* Dummy security context for avoiding NULL pointer dereference. */
-extern struct ccs_security ccs_oom_security;
+static struct ccs_security ccs_oom_security = {
+        .ccs_domain_info = &ccs_kernel_domain
+};
+
 /* Dummy security context for avoiding NULL pointer dereference. */
-extern struct ccs_security ccs_default_security;
+static struct ccs_security ccs_default_security = {
+        .ccs_domain_info = &ccs_kernel_domain
+};
+
+/* List of "struct ccs_security". */
+struct list_head ccs_task_security_list[CCS_MAX_TASK_SECURITY_HASH];
+/* Lock for protecting ccs_task_security_list[]. */
+static DEFINE_SPINLOCK(ccs_task_security_list_lock);
+
 /* Dummy marker for calling security_bprm_free(). */
 static const unsigned long ccs_bprm_security;
 
@@ -131,13 +145,13 @@ static void ccs_clear_execve(int ret, struct ccs_security *security)
  */
 static int ccs_task_alloc_security(struct task_struct *p)
 {
-	int rc = ccs_alloc_task_security(p);
+	int rc = __ccs_alloc_task_security(p);
 	if (rc)
 		return rc;
 	while (!original_security_ops.task_alloc_security);
 	rc = original_security_ops.task_alloc_security(p);
 	if (rc)
-		ccs_free_task_security(p);
+		__ccs_free_task_security(p);
 	return rc;
 }
 
@@ -152,7 +166,7 @@ static void ccs_task_free_security(struct task_struct *p)
 {
 	while (!original_security_ops.task_free_security);
 	original_security_ops.task_free_security(p);
-	ccs_free_task_security(p);
+	__ccs_free_task_security(p);
 }
 
 /**
@@ -1367,7 +1381,7 @@ static int __init ccs_init(void)
 		goto out;
 	ccs_main_init();
 	ccs_update_security_ops(ops);
-	printk(KERN_INFO "AKARI: 1.0.35   2015/11/11\n");
+	printk(KERN_INFO "AKARI: 1.0.36   2017/02/20\n");
 	printk(KERN_INFO
 	       "Access Keeping And Regulating Instrument registered.\n");
 	return 0;
@@ -1391,3 +1405,142 @@ bool ccs_used_by_cred(const struct ccs_domain_info *domain)
 {
 	return false;
 }
+
+/**
+ * ccs_add_task_security - Add "struct ccs_security" to list.
+ *
+ * @ptr:  Pointer to "struct ccs_security".
+ * @list: Pointer to "struct list_head".
+ *
+ * Returns nothing.
+ */
+static void ccs_add_task_security(struct ccs_security *ptr,
+				  struct list_head *list)
+{
+	unsigned long flags;
+	spin_lock_irqsave(&ccs_task_security_list_lock, flags);
+	list_add_rcu(&ptr->list, list);
+	spin_unlock_irqrestore(&ccs_task_security_list_lock, flags);
+}
+
+/**
+ * __ccs_alloc_task_security - Allocate memory for new tasks.
+ *
+ * @task: Pointer to "struct task_struct".
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int __ccs_alloc_task_security(const struct task_struct *task)
+{
+	struct ccs_security *old_security = ccs_current_security();
+	struct ccs_security *new_security = kzalloc(sizeof(*new_security),
+						    GFP_KERNEL);
+	struct list_head *list = &ccs_task_security_list
+		[hash_ptr((void *) task, CCS_TASK_SECURITY_HASH_BITS)];
+	if (!new_security)
+		return -ENOMEM;
+	new_security->task = task;
+	new_security->ccs_domain_info = old_security->ccs_domain_info;
+	new_security->ccs_flags = old_security->ccs_flags;
+	ccs_add_task_security(new_security, list);
+	return 0;
+}
+
+/**
+ * ccs_find_task_security - Find "struct ccs_security" for given task.
+ *
+ * @task: Pointer to "struct task_struct".
+ *
+ * Returns pointer to "struct ccs_security" on success, &ccs_oom_security on
+ * out of memory, &ccs_default_security otherwise.
+ *
+ * If @task is current thread and "struct ccs_security" for current thread was
+ * not found, I try to allocate it. But if allocation failed, current thread
+ * will be killed by SIGKILL. Note that if current->pid == 1, sending SIGKILL
+ * won't work.
+ */
+struct ccs_security *ccs_find_task_security(const struct task_struct *task)
+{
+	struct ccs_security *ptr;
+	struct list_head *list = &ccs_task_security_list
+		[hash_ptr((void *) task, CCS_TASK_SECURITY_HASH_BITS)];
+	/* Make sure INIT_LIST_HEAD() in ccs_mm_init() takes effect. */
+	while (!list->next);
+	rcu_read_lock();
+	list_for_each_entry_rcu(ptr, list, list) {
+		if (ptr->task != task)
+			continue;
+		rcu_read_unlock();
+		return ptr;
+	}
+	rcu_read_unlock();
+	if (task != current)
+		return &ccs_default_security;
+	/* Use GFP_ATOMIC because caller may have called rcu_read_lock(). */
+	ptr = kzalloc(sizeof(*ptr), GFP_ATOMIC);
+	if (!ptr) {
+		printk(KERN_WARNING "Unable to allocate memory for pid=%u\n",
+		       task->pid);
+		send_sig(SIGKILL, current, 0);
+		return &ccs_oom_security;
+	}
+	*ptr = ccs_default_security;
+	ptr->task = task;
+	ccs_add_task_security(ptr, list);
+	return ptr;
+}
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 8)
+
+/**
+ * ccs_rcu_free - RCU callback for releasing "struct ccs_security".
+ *
+ * @rcu: Pointer to "struct rcu_head".
+ *
+ * Returns nothing.
+ */
+static void ccs_rcu_free(struct rcu_head *rcu)
+{
+	struct ccs_security *ptr = container_of(rcu, typeof(*ptr), rcu);
+	kfree(ptr);
+}
+
+#else
+
+/**
+ * ccs_rcu_free - RCU callback for releasing "struct ccs_security".
+ *
+ * @arg: Pointer to "void".
+ *
+ * Returns nothing.
+ */
+static void ccs_rcu_free(void *arg)
+{
+	struct ccs_security *ptr = arg;
+	kfree(ptr);
+}
+
+#endif
+
+/**
+ * __ccs_free_task_security - Release memory associated with "struct task_struct".
+ *
+ * @task: Pointer to "struct task_struct".
+ *
+ * Returns nothing.
+ */
+static void __ccs_free_task_security(const struct task_struct *task)
+{
+	unsigned long flags;
+	struct ccs_security *ptr = ccs_find_task_security(task);
+	if (ptr == &ccs_default_security || ptr == &ccs_oom_security)
+		return;
+	spin_lock_irqsave(&ccs_task_security_list_lock, flags);
+	list_del_rcu(&ptr->list);
+	spin_unlock_irqrestore(&ccs_task_security_list_lock, flags);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 8)
+	call_rcu(&ptr->rcu, ccs_rcu_free);
+#else
+	call_rcu(&ptr->rcu, ccs_rcu_free, ptr);
+#endif
+}
