@@ -3,12 +3,14 @@
  *
  * Copyright (C) 2010-2015  Tetsuo Handa <penguin-kernel@I-love.SAKURA.ne.jp>
  *
- * Version: 1.0.35   2015/11/11
+ * Version: 1.0.36   2017/02/20
  */
 
 #include <linux/version.h>
 #include <linux/security.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 7, 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 12, 0)
+#include "lsm-4.12.c"
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(4, 7, 0)
 #include "lsm-4.7.c"
 #elif LINUX_VERSION_CODE >= KERNEL_VERSION(4, 2, 0)
 #include "lsm-4.2.c"
