@@ -9,6 +9,9 @@
 #ifndef __init
 #include <linux/init.h>
 #endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 11, 0)
+#include <linux/cred.h>
+#endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 #error This module supports only 2.6.0 and later kernels.
@@ -26,7 +29,7 @@
 #error You must choose CONFIG_MODULES=y for building this module.
 #endif
 
-#if 0
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 1, 0) && !defined(SECURITY_NAME_MAX)
 #include <linux/lsm_hooks.h>
 extern struct security_hook_heads probe_dummy_security_hook_heads;
 struct security_hook_heads * __init probe_security_hook_heads(void);
